source [file dirname $argv0]/setup.tcl
source [file dirname $argv0]/topology.tcl
source [file dirname $argv0]/flowgen.tcl
source [file dirname $argv0]/flowmon.tcl
source [file dirname $argv0]/config.tcl

set_defaults
getopt $argc $argv
set_defaults_post

set setABE 0
if { $opt(bqueue) == "ABE" } {
  set opt(bqueue) "DSF"
  set setABE 1
}

if { $opt(quiet) == 0 } {
	puts -nonewline "simulation setup"; flush stdout
}

set topo [new Dumbbell $config(conns)]
set bq [$topo bottleneck-queue]

if { $opt(bqueue) == "DSF" || $opt(bqueue) == "FIFO" || $opt(bqueue) == "SPRI" || $opt(bqueue) == "VIFQ" } {
	if { $opt(bqueue) != "DSF" } {
		$bq set-queue-limit $opt(maxdelay)ms
	}
	if { $opt(bqueue) != "SPRI" } {
		for {set i 0} {$i < $opt(tgroups)} {incr i} {
			if [info exists opt(dtarget$i)] {
				$bq add-class $i [set opt(dtarget$i)]ms
			}
		}
	}
	if { $setABE == 1 } {
	  $bq set-abe
	}
	if { [info exists linktracefile] } { $bq attach $linktracefile }
} else {
	puts [format "ERROR - unknown queue type: %s" $opt(bqueue)]
	exit 1
}

if { [info exists flowtracefile] } {
	flow-mon [$topo bottleneck-link] 1.0 $flowtracefile
}

#set FlowgenDebug 1

set cnt 0
for {set i 0} {$i < $opt(tgroups)} {incr i} {
	set relload [expr $opt(load$i) / $opt(connections$i)]
	set bspeedbits [expr $opt(bspeed) * 1e+6]
	set absload [expr $relload * $bspeedbits]
	set overhead [expr ($opt(psize$i) * 4.0) / $bspeedbits]
#	set overhead 0
	set endnodes [$topo end-nodes-pair $i]
	if [info exists opt(dtarget$i)] {
		set sclass $i
	} else {
		set sclass $opt(sclass$i)
	}
	if { $opt(traffic$i) == "ShortWeb" } {
		set iat_avg [expr 10.0 / $opt(connections$i)]
		set f($i) [new FlowSet "flow[lindex $endnodes 0][lindex $endnodes 1]"]
		$f($i) setup $endnodes $i $opt(connections$i) $sclass TCP File \
			[list $opt(psize$i) $overhead [expr $config(filesize)/$opt(psize$i)]]
		$f($i) setFlowArrival $iat_avg exp 0 fixed
		if { [info exists comptracefile] } { $f($i) attach $comptracefile }
		$ns at $opt(start$i) "$f($i) start"
		$ns at $opt(stop$i)  "$f($i) stop"
		set cnt [expr $cnt + 1]
	} else {
		for {set j 0} {$j < $opt(connections$i)} {incr j} {
			if { $opt(traffic$i) == "TCP" } {
				set f($cnt) [new Flow "flow[lindex $endnodes 0][lindex $endnodes 1]"]
				$f($cnt) setup $endnodes $i $sclass TCP Greedy \
					[list $opt(psize$i) $overhead]
			} elseif { $opt(traffic$i) == "TCPX" } {
				set f($cnt) [new Flow "flow[lindex $endnodes 0][lindex $endnodes 1]"]
				$f($cnt) setup $endnodes $i $sclass TCPX Greedy \
					[list $opt(psize$i) $overhead]
			} elseif { $opt(traffic$i) == "TFRC" } {
				set f($cnt) [new Flow "flow[lindex $endnodes 0][lindex $endnodes 1]"]
				$f($cnt) setup $endnodes $i $sclass TFRC Greedy \
					[list [expr $opt(psize$i) - 32]]
			} elseif { $opt(traffic$i) == "CBR" } {
				set f($cnt) [new Flow "flow[lindex $endnodes 0][lindex $endnodes 1]"]
				$f($cnt) setup $endnodes $i $sclass UDP CBR \
					[list $opt(psize$i) $absload 1]
			} elseif { $opt(traffic$i) == "Poisson" } {
				set f($cnt) [new Flow "flow[lindex $endnodes 0][lindex $endnodes 1]"]
				$f($cnt) setup $endnodes $i $sclass UDP Poisson \
					[list $opt(psize$i) $bspeedbits $absload]
			} elseif { $opt(traffic$i) == "Pareto" } {
				set ptime [expr 8.0 * $opt(psize$i) / $absload]
				set f($cnt) [new Flow "flow[lindex $endnodes 0][lindex $endnodes 1]"]
				$f($cnt) setup $endnodes $i $sclass UDP Pareto \
					[list $opt(psize$i) [expr 2 * $absload] [expr $ptime]s [expr $ptime]s $opt(shape$i)]
			} elseif { $opt(traffic$i) == "ShortSimple" } {
				set f($cnt) [new FlowSet "flow[lindex $endnodes 0][lindex $endnodes 1]"]
				$f($cnt) setup $endnodes $i 1 $sclass TCP File \
					[list $opt(psize$i) $overhead [expr $config(filesize)/$opt(psize$i)]]
				$f($cnt) setFlowArrival 0 fixed 0 fixed
			} else {
				puts [format "ERROR - unknown traffic type %s for class %d" $opt(traffic$i) $i]
				exit 1
			}
			$ns at [expr $opt(start$i) + $opt(spread$i) * $j / $opt(connections$i)] "$f($cnt) start"
			$ns at $opt(stop$i)  "$f($cnt) stop"
			incr cnt
		}
	}
}

if { $opt(quiet) == 0 } {
	showProgress $opt(stopTime)
}
$ns at $opt(stopTime) "finish"

if { $opt(quiet) == 0 } {
	puts -nonewline "\rsimulation start"; flush stdout
}
$ns run
