# helper class to create a topology

Class Topology

Topology instproc init { } {}

Topology instproc setTopologyLink { l o bw d } {
	$self instvar links orientations bandwidths delays
	lappend links $l
	lappend orientations $o
	lappend bandwidths $bw
	lappend delays $d
}

Topology instproc createTopology {nodecount} {
	global ns
	$self instvar links orientations bandwidths delays nodes

	for {set i 0} { $i < $nodecount } {incr i} { set nodes($i) [$ns node] }
	for {set i 0} { $i < [llength $links] } {incr i} {
		set linkdata [lindex $links $i]
		set a [lindex $linkdata 0]
		set b [lindex $linkdata 1]
		if ![info exists nodes($a)] { set nodes($a) [$ns node] }
		if ![info exists nodes($b)] { set nodes($b) [$ns node] }

		set bandwidth 10Mb
		if { [lindex $bandwidths $i] != "" } { set bandwidth [lindex $bandwidths $i] }
		set delay 10ms
		if { [lindex $delays $i] != "" } { set delay [lindex $delays $i] }
		set queueType "DropTail"
		if { [lindex $linkdata 2] != "" } { set queueType  [lindex $linkdata 2] }
		set queueType2 $queueType
		if { [lindex $linkdata 3] != "" } { set queueType2 [lindex $linkdata 3] }

		$ns simplex-link $nodes($a) $nodes($b) $bandwidth $delay $queueType
		$ns simplex-link $nodes($b) $nodes($a) $bandwidth $delay $queueType2

		if { [lindex $orientations $i] != "" } {
			switch [lindex $orientations $i] {
				NW { set o "left-up" }
				N  { set o "up" }
				NE { set o "right-up" }
				E  { set o "right" }
				SE { set o "right-down" }
				S  { set o "down" }
				SW { set o "left-down" }
				W  { set o "left" }
				default { set o "right" }
			}
			$ns duplex-link-op $nodes($a) $nodes($b) orient $o
		}
	}
}

Topology instproc node { i } {
	$self instvar nodes
	return $nodes($i)
}

#
# create topology like 
# \           /     multiple connections
#  \         /
# --+-------+-----
#  /         \
# /           \
#
# source topology.tcl

Class Dumbbell

##################################
# creating the topology
# idea: we need 2 * access links + 2 nodes
# last two are the bottleneck link nodes
# node from 0 to alinks-1 connect to head end of bottleneck
# node from alinks-1 to 2*alinks-1-1 connect to tail end of bottleneck
##################################
Dumbbell instproc init { conns } {
	global opt
	$self instvar alinks_ mytopo
	set mytopo [new Topology]
	set alinks_ $conns
	set bhead [expr $alinks_*2]
	set btail [expr $alinks_*2+1]
	set blink [list $bhead $btail $opt(bqueue) DropTail]
	$mytopo setTopologyLink $blink E [set opt(bspeed)]Mb [set opt(bdelay)]ms
	for {set i 0} {$i < $alinks_} {incr i} {
#		puts [format "link %d <-> %d and %d <-> %d" $i $bhead $btail [expr $i + $alinks_]]
		set headlink [list $i $bhead]
		$mytopo setTopologyLink $headlink E [set opt(aspeed$i)]Mb [set opt(adelay$i)]ms
		set taillink [list $btail [expr $i + $alinks_]]
		$mytopo setTopologyLink $taillink E [set opt(aspeed$i)]Mb [set opt(adelay$i)]ms
	}
	$mytopo createTopology [expr $alinks_*2+2]
}

# get the bottleneck nodes
Dumbbell instproc bottleneck-nodes {} {
	$self instvar alinks_ mytopo
#	puts [format "bottleneck nodes: %d <-> %d, check: %d <-> %d" [expr $alinks_*2] [expr $alinks_*2+1] [[$mytopo node [expr $alinks_*2]] id] [[$mytopo node [expr $alinks_*2+1]] id] ]
	return [list [$mytopo node [expr $alinks_*2]] [$mytopo node [expr $alinks_*2+1]]]
}

# get the bottleneck link
Dumbbell instproc bottleneck-link {} {
	global ns
	set bnodes [$self bottleneck-nodes]
	return [$ns link [lindex $bnodes 0] [lindex $bnodes 1]]
}

# get the bottleneck queue
Dumbbell instproc bottleneck-queue {} {
	return [[$self bottleneck-link] queue]
}

# get the pair of the end to end nodes
Dumbbell instproc end-nodes-pair { id } {
	$self instvar alinks_ mytopo
	return [list [$mytopo node $id] [$mytopo node [expr $id + $alinks_]]]
}
